/* Test.c
** Test application to demonstrate
** SetThreadAbsolutePriority
**
** Author: Jeff Claar
*/

#define WIN32_LEAN_AND_MEAN
#define STRICT
#define _WIN32_WINNT 0x0400   /* For QueueUserAPC */
#include <windows.h>
#include <stdio.h>
#include "priority.h"

/* Priorities for threads */
KPRIORITY pr[] = { 16, 20, 24, 28, 31 };
#define NUM_PRIORITIES  (sizeof(pr) / sizeof(pr[0]))

HANDLE hThreads[NUM_PRIORITIES];

/* Event to tell threads to terminate. */
HANDLE hTerminateEvent;

DWORD CALLBACK sThreadProc(LPVOID pData);
VOID CALLBACK sAPC(DWORD);

void main()
{
    DWORD id;
    int i;

    hTerminateEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

    /*
    ** Create each of the threads, and pass the thread's
    ** priority as the thread argument.
    */
    for (i = 0; i < NUM_PRIORITIES; i++)
    {
        hThreads[i] = CreateThread(
         NULL,
         0,
         sThreadProc,
         (LPVOID) pr[i],
         0,
         &id);
    }

    i = 0;
    printf("Press <ENTER> to terminate.\n");

    /* Clear the state of the enter key. */
    GetAsyncKeyState(VK_RETURN);
    while ((GetAsyncKeyState(VK_RETURN) & 1) == 0)
    {
        Sleep(1000);
        /*
        ** Queue an APC to each of the threads.
        ** This causes the thread to wake up
        ** and would normally cause a dynamic
        ** boost of the thread's priority.
        */
        QueueUserAPC(sAPC, hThreads[i], 0);
        i++;
        i %= NUM_PRIORITIES;
    }

    /*
    ** User pressed ENTER. Set the event and
    ** wait for the threads to terminate.
    */
    SetEvent(hTerminateEvent);
    WaitForMultipleObjects(NUM_PRIORITIES, hThreads,
        TRUE, INFINITE);

    /* Clean up */
    for (i = 0; i < NUM_PRIORITIES; i++)
    {
        CloseHandle(hThreads[i]);
    }

    CloseHandle(hTerminateEvent);
}

DWORD CALLBACK sThreadProc(LPVOID pData)
{
    /*
    ** Set the thread's priority to the passed in
    ** value.
    */
    SetThreadAbsolutePriority(GetCurrentThread(),
        (KPRIORITY) pData);

    /*
    ** Wait for the termination event to be
    ** signalled. If an APC is received, just
    ** print the thread ID and go back to sleep.
    */
    while (WaitForSingleObjectEx(hTerminateEvent,
        INFINITE, TRUE) == WAIT_IO_COMPLETION)
    {
        printf("Thread %d Received APC\n", GetCurrentThreadId());
    }
    return 0;
}

VOID CALLBACK sAPC(DWORD test)
{
    /*
    ** Dummy function to process APCs.
    ** The APC is only necessary to cause
    ** the thread to wake up.
    */
}
