#include <basedef.h>
#include <vmm.h>
#include <debug.h>
#include <vxdwraps.h>
#include <vwin32.h>
#include "ndisioct.h"

void *  __cdecl memcpy(void *, const void *, int);
#pragma intrinsic(memcpy)

// functions in asm module
void Device1ControlProc( void );
void Device2ControlProc( void );

// functions in MP_STUB.C
#define NDIS_API  __stdcall
ULONG NDIS_API MiniportQueryInformation(      
    void *MiniportAdapterContext,
    ULONG Oid,
    void *InformationBuffer,
    ULONG InformationBufferLength,
    ULONG *BytesWritten,
    ULONG *BytesNeeded
);

#define NUM_DEVICES     2
#define OID_802_3_CURRENT_ADDRESS           	0x01010102

typedef struct VxD_Desc_Block DDB;
typedef struct
{
   DDB   ddb;
   void  *MiniportContext;
} DEVICE_INSTANCE;

DDB templateDDB = 
{ 
   0,                      // link to next DDB
   4,                      // VxD built with this version of DDK
   UNDEFINED_DEVICE_ID,    // no V86 or PM API, don't need ID
   4, 0,                   // our version number, major and minor
   0,                      // flags: init to zero
   // 8 byte name, trailing spaces if needed 
   {'M', 'P', 'O', 'R', 'T', '_', 'D', 'x'},    
   UNDEFINED_INIT_ORDER    // determines place in DDB chain
};

DEVICE_INSTANCE aDevices[ NUM_DEVICES ];
ULONG aControlProcs[ NUM_DEVICES ] = 
{
   Device1ControlProc, 
   Device2ControlProc, 
};
char dbgbuf[100];
DDB * VMM_GetDDBList( void );

void TraverseDDBList( void )
{
   DDB   *node = VMM_GetDDBList();
   int   i;

   while (node)
   {
      _Sprintf(dbgbuf,"%c%c%c%c%c%c%c%c\n", 
               node->DDB_Name[0],node->DDB_Name[1],node->DDB_Name[2],node->DDB_Name[3], 
               node->DDB_Name[4],node->DDB_Name[5],node->DDB_Name[6],node->DDB_Name[7]);
      Out_Debug_String(dbgbuf);
      node = (DDB *)node->DDB_Next;
   }
}

BOOL Register_VxD_DDB( int DeviceNum, void *Context )
{
   int   ix = DeviceNum-1;
BOOL rc;

   aDevices[ix].MiniportContext = Context;
   memcpy( &aDevices[ix].ddb, &templateDDB, sizeof(DDB) );
   aDevices[ix].ddb.DDB_Control_Proc = aControlProcs[ix];
   aDevices[ix].ddb.DDB_Name[7] = '0'+DeviceNum;
   rc = VMM_Add_DDB( &aDevices[ix].ddb );
   TraverseDDBList();
   return rc;
}

DDB * VMM_GetDDBList( void )
{
   VMMCall( VMM_GetDDBList );
}

BOOL VMM_Add_DDB(PVMMDDB pDDB)
{
   Touch_Register(EDI)
   _asm mov edi, pDDB
   VMMCall( VMM_Add_DDB );
   _asm jnc ok
   return FALSE;
ok:
   return TRUE;
}

BOOL VMM_Remove_DDB(PVMMDDB pDDB)
{
   Touch_Register(EDI)
   _asm mov edi, pDDB
   VMMCall( VMM_Remove_DDB );
   _asm jnc ok
   return FALSE;
ok:
   return TRUE;
}

DWORD OnW32DeviceIoControl(ULONG DeviceNum, PDIOCPARAMETERS p)
{
   ULONG    cbRead, cbNeeded;
   ULONG    Status;

   switch (p->dwIoControlCode)
   {
   case DIOC_OPEN:   
      return 0;

   case DIOC_CLOSEHANDLE:  // file closed
      return 0;

   case NDISIOCTL_GET_MACADDRESS:
      Status = MiniportQueryInformation( 
         aDevices[DeviceNum].MiniportContext,  
         OID_802_3_CURRENT_ADDRESS, 
         p->lpvOutBuffer, 
         p->cbOutBuffer, 
         &cbRead, 
         &cbNeeded );
      return Status;

   case NDISIOCTL_SET_MACADDRESS:
      return 0;
   }
}



